package ast

import "git.akyoto.dev/cli/q/src/expression"

// Node is an interface used for all types of AST nodes.
type Node any

// AST is an abstract syntax tree which is simply a list of nodes.
type AST []Node

// Assert is a condition that must be true, otherwise the program stops.
type Assert struct {
	Condition *expression.Expression
}

// Assign is an assignment to an existing variable or memory location.
type Assign struct {
	Expression *expression.Expression
}

// Call is a function call.
type Call struct {
	Expression *expression.Expression
}

// Case is a case inside a switch.
type Case struct {
	Condition *expression.Expression
	Body      AST
}

// Define is a variable definition.
type Define struct {
	Expression *expression.Expression
}

// For is a loop with a defined iteration limit.
type For struct {
	Head *expression.Expression
	Body AST
}

// If is a conditional branch.
type If struct {
	Condition *expression.Expression
	Body      AST
	Else      AST
}

// Loop is an infinite loop.
type Loop struct {
	Body AST
}

// Return is a return statement.
type Return struct {
	Values []*expression.Expression
}

// Switch is a conditional branch with multiple cases.
type Switch struct {
	Cases []Case
}
