package ast

import (
	"git.akyoto.dev/cli/q/src/errors"
	"git.akyoto.dev/cli/q/src/expression"
	"git.akyoto.dev/cli/q/src/fs"
	"git.akyoto.dev/cli/q/src/token"
)

// parseInstruction generates an AST node from an instruction.
func parseInstruction(tokens token.List, file *fs.File, nodes AST) (Node, error) {
	if tokens[0].IsKeyword() {
		return parseKeyword(tokens, file, nodes)
	}

	expr := expression.Parse(tokens)

	if expr == nil {
		return nil, nil
	}

	switch {
	case IsFunctionCall(expr):
		return &Call{Expression: expr}, nil

	case IsVariableDefinition(expr):
		if len(expr.Children) < 2 {
			return nil, errors.New(errors.MissingOperand, file, expr.Token.End())
		}

		return &Define{Expression: expr}, nil

	case IsAssignment(expr):
		if len(expr.Children) < 2 {
			return nil, errors.New(errors.MissingOperand, file, expr.Token.End())
		}

		return &Assign{Expression: expr}, nil

	default:
		return nil, errors.New(&errors.InvalidInstruction{Instruction: tokens.Text(file.Bytes)}, file, tokens[0].Position)
	}
}
