package core

import (
	"git.akyoto.dev/cli/q/src/asm"
	"git.akyoto.dev/cli/q/src/errors"
	"git.akyoto.dev/cli/q/src/expression"
	"git.akyoto.dev/cli/q/src/types"
)

// CompileDelete compiles a `delete` function call which deallocates a struct.
func (f *Function) CompileDelete(root *expression.Expression) error {
	parameters := root.Children[1:]
	variableName := parameters[0].Token.Text(f.File.Bytes)
	variable := f.VariableByName(variableName)

	if variable == nil {
		return errors.New(&errors.UnknownIdentifier{Name: variableName}, f.File, parameters[0].Token.Position)
	}

	defer f.UseVariable(variable)
	f.SaveRegister(f.CPU.Input[0])
	f.SaveRegister(f.CPU.Input[1])
	f.RegisterRegister(asm.MOVE, f.CPU.Input[0], variable.Register)
	f.RegisterNumber(asm.MOVE, f.CPU.Input[1], variable.Type.(*types.Pointer).To.Size())
	f.CallSafe(f.Functions["mem.free"], f.CPU.Input[:2])
	return nil
}
